import { defineStore } from "pinia";
import { useConnectionStore } from "./connection";
import * as Y from "yjs";
import { Awareness } from "y-protocols/awareness.js";
import { computed, ref } from "vue";

export const useMissionDetailStore = defineStore("missionDetail", {
  state: () => {
    return {
      yDoc: new Y.Doc(),
      docId: null as null | string,
      lastUpdateTimestamp: 0 as number,
      connectionStatus: "disconnected", // 'disconnected', 'connecting', 'connected', 'syncing', 'synced'
    };
  },
  actions: {
    initialize(docId: string) {
      this.docId = docId;

      this.lastUpdateTimestamp = this.loadLastUpdateFromLocalStorage();

      this.setupSocketHandlers();
      this.setupYjsObservers();
    },

    setupSocketHandlers() {
      const connectionStore = useConnectionStore();
      if (!connectionStore.connection) return;

      connectionStore.connection.on("package-sync", (data) => {
        try {
          this.connectionStatus = "syncing";

          Y.applyUpdate(this.yDoc, new Uint8Array(data.update));

          if (data.timestamp > this.lastUpdateTimestamp) {
            this.lastUpdateTimestamp = data.timestamp;
            this.saveLastUpdateToLocalStorage();
          }

          this.connectionStatus = "synced";
        } catch (error) {
          console.error("Error applying update:", error);
          this.requestFullSync();
        }
      });
      connectionStore.connection.on("sync-get-missing-updates", (data) => {
        const clientUpdates = Y.encodeStateAsUpdate(this.yDoc, new Uint8Array(data.stateVector));

        connectionStore.connection?.emit("mission:sync-client-updates", {
          update: clientUpdates,
          timestamp: Date.now(),
        });
      });

      this.joinDocument();
    },

    setupYjsObservers() {
      if (!this.yDoc) return;

      this.yDoc.on("update", (update) => {
        const connectionStore = useConnectionStore();
        if (connectionStore.connected) {
          connectionStore.connection?.emit("mission:sync-client-updates", {
            update: Array.from(update),
            timestamp: Date.now(),
          });
        }

        this.lastUpdateTimestamp = Date.now();
        this.saveLastUpdateToLocalStorage();
      });
    },

    joinDocument() {
      const connectionStore = useConnectionStore();
      if (!connectionStore.connection || !this.docId) return;

      connectionStore.connection.emit("mission:join", this.docId, {
        timestamp: this.lastUpdateTimestamp,
      });
    },

    requestFullSync() {
      const connectionStore = useConnectionStore();
      if (!connectionStore.connection || !this.docId) return;

      connectionStore.connection.emit("mission:join", this.docId, null);
    },

    loadLastUpdateFromLocalStorage() {
      if (!this.docId) return 0;

      const stored = localStorage.getItem(`yjsDoc_timestamp`);
      return stored ? parseInt(stored, 10) : 0;
    },

    saveLastUpdateToLocalStorage() {
      if (!this.docId) return;

      localStorage.setItem(`yjsDoc_timestamp`, this.lastUpdateTimestamp.toString());
    },

    cleanup() {
      if (this.yDoc) {
        this.yDoc.destroy();
        this.yDoc = new Y.Doc();
      }
      this.lastUpdateTimestamp = 0;
      localStorage.removeItem("yjsDoc_timestamp");

      const connectionStore = useConnectionStore();
      connectionStore.connection?.emit("mission:leave");

      this.connectionStatus = "disconnected";
    },
  },
});
